"""File"""
import json,codecs,configparser,subprocess,platform,os,glob,shaonutil

def get_all_functions(object):
	"""shaonutil.file.get_all_functions(object/file/class)"""
	return [func for func in dir(object) if callable(getattr(object, func))]

def ConfigSectionMap(Config, section):
    dict1 = {}
    options = Config.options(section)
    for option in options:
        try:
            dict1[option] = Config.get(section, option)
            if dict1[option] == -1:
                DebugPrint("skip: %s" % option)
        except:
            print("exception on %s!" % option)
            dict1[option] = None
    return dict1

class CaseConfigParser(configparser.SafeConfigParser):
    def optionxform(self, optionstr):
        return optionstr

def read_configuration_ini(filename):
	config = configparser.ConfigParser()
	config.readfp(codecs.open(filename, "r", "utf8"))
	return config

def read_safecase_configuration_ini(filename):
	config = CaseConfigParser()
	config.readfp(codecs.open(filename, "r", "utf8"))
	return config

def write_configuration_ini(configs_par,filename, f_mode='w'):
	with open(filename, f_mode) as configfile:    # save
		configs_par.write(configfile)

def read_json(filename):
	"""Read JSON file and return dictionary"""
	with codecs.open(filename, "r", encoding="utf-8") as fp:
		data = json.load(fp)
	#Print Formatted Dictionary
	#print(json.dumps(data, indent=4))
	return data

def write_json(obj,filename):
	"""Write JSON file"""
	with codecs.open(filename, "w", encoding='utf-8') as fp:
	    json.dump(obj, fp, indent=1)

def read_file(filename):
	"""Read File and return lines as list"""
	with codecs.open(filename, "r", encoding="utf-8") as file_reader:
		lines = file_reader.readlines()

	ill_chars = ['\r','\n']
	_ = []
	for line in lines:
		for ic in ill_chars:
			line = line.replace(ic,'')
		_.append(line)
	filtered_lines = _
	return filtered_lines

def write_file(filename, strs,mode="w"):
	"""Write File from string"""
	with codecs.open(filename, mode, encoding='utf-8') as file_appender:
		file_appender.writelines(strs)

def read_pickle(filename):
	# Unpickling
	with open(filename, "rb") as fp:
		b = pickle.load(fp)
	return b

def write_pickle(filename, obj_):
	with open(filename, "wb") as fp:   #Pickling
		pickle.dump(obj_, fp)


def open_file_with_default_app(filepath):
	if platform.system() == 'Darwin':       # macOS
	    subprocess.call(('open', filepath))
	elif platform.system() == 'Windows':    # Windows
	    os.startfile(filepath)
	elif platform.system() == 'Windows':    # Windows
		subprocess.call(('xdg-open', filepath))
	else:                                   # linux variants
	    subprocess.call(('xdg-open', filepath))

def get_last_file_of_dir(filename):
	list_of_files = glob.glob(filename)
	latest_file = max(list_of_files, key=os.path.getctime)
	return latest_file

def remove_duplicateLines_from_file(filename):
	lines = shaonutil.file.read_file(filename)
	lines_seen = set() # holds lines already seen

	for line in lines:
		if line not in lines_seen: # not a duplicate
			lines_seen.add(line)

	strs = '\n'.join(lines_seen)
	shaonutil.file.write_file(filename, strs)

if __name__ == '__main__':
	pass


"""String"""
import string
import random
import pprint
import uuid

def nicely_print(dictionary,print=True):
	"""Prints the nicely formatted dictionary - shaonutil.strings.nicely_print(object)"""
	if print: pprint.pprint(dictionary)

	# Sets 'pretty_dict_str' to 
	return pprint.pformat(dictionary)

def change_dic_key(dic,old_key,new_key):
	"""Change dictionary key with new key"""
	dic[new_key] = dic.pop(old_key)
	return dic

def randomString(stringLength=10):
    """Generate a random string of fixed length """
    letters = string.ascii_lowercase
    for c in range(10):
        letters = letters + str(c)

    return ''.join(random.choice(letters) for i in range(stringLength))

def generateSecureRandomString(stringLength=10):
    """Generate a secure random string of letters, digits and special characters """
    password_characters = string.ascii_letters + string.digits #+ string.punctuation
    return ''.join(secrets.choice(password_characters) for i in range(stringLength))

def generateCryptographicallySecureRandomString(stringLength=10):
	"""Generate a random string in a UUID fromat which is crytographically secure and random"""
	randomString = uuid.uuid4().hex.upper() 
	randomString  = randomString[0:stringLength]
	return randomString

if __name__ == '__main__':
	pass


"""Network"""
from io import StringIO
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import  shaonutil
import urllib.parse
import re


def url_encoding_to_utf_8(url):
	"""url_encoding_to_utf_8(url)"""
	url = urllib.parse.unquote(url)
	return url


def check_valid_url(url):
	regex = re.compile(
	        r'^(?:http|ftp)s?://' # http:// or https://
	        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|' #domain...
	        r'localhost|' #localhost...
	        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
	        r'(?::\d+)?' # optional port
	        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

	return re.match(regex, url) is not None


class Email:
	def __init__(self):
		self._authentication = ''

	@property
	def authentication(self):
		## changing the argument values by accesing property via setter methods
		return self._authentication

	@authentication.setter
	def authentication(self, new_value):
		self._authentication = new_value

	def send_email(self,receiver_address,subject,mail_content,attachment_file_link,log=False):
		#The mail addresses and password
		smtp_server = self._authentication['smtp_server']
		smtp_port = self._authentication['smtp_port']
		sender_address = self._authentication['smtp_username']
		sender_pass = self._authentication['smtp_password']

		receiver_address = receiver_address

		#Setup the MIME
		message = MIMEMultipart()
		message['From'] = sender_address
		message['To'] = receiver_address
		message['Subject'] = subject
		#The body and the attachments for the mail
		message.attach(MIMEText(mail_content, 'plain'))
		#Create SMTP session for sending the mail
		session = smtplib.SMTP(smtp_server, smtp_port) #use gmail with port
		session.starttls() #enable security

		try:
			session.login(sender_address, sender_pass) #login with mail_id and password
		except smtplib.SMTPAuthenticationError as e:
			#: (535, b'5.7.8 Username and Password not accepted. Learn more at\n5.7.8  https://support.google.com/mail/?p=BadCredentials j24sm19239050pfi.55 - gsmtp')
			if(log): print('if using gmail smtp, check server host,port\n  if user/pass is not accepeted , turn on less secure app setting and recognize the activity warning mail sent in your gmail, then it will send email in next run.')
			stat_message = str(e)
			return stat_message


		text = message.as_string()
		session.sendmail(sender_address, receiver_address, text)
		session.quit()
		stat_message = 'Mail Sent'
		return stat_message


if __name__ == '__main__':
	pass